/*==========================================================*/
/*  TetrahedraPair.c                                        */
/*==========================================================*/
/*                                                          */
/*  A pair of rotating tetrahedra with color interpolation  */
/*                                                          */
/*  Demonstration of use of homogeneous coordinate          */
/*  transformations and a simple data structure             */
/*                                                          */
/*  Colors are assigned to the vertices                     */
/*                                                          */
/*  Author:  Bary W Pollack                                 */
/*  Updated: Apr. 7, 2005                                   */
/*                                                          */
/*==========================================================*/

#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>

/*==========================================================*/

#define dX0 -2.0            // offset from the origin

GLfloat verticesL[][3] =
{
    {-dX0+1.0, -1.0, -1.0}, 
    {-dX0-1.0, +1.0, -1.0}, 
    {-dX0+1.0, +1.0, +1.0}, 
    {-dX0-1.0, -1.0, +1.0}
};

GLfloat verticesR[][3] =
{
    {+dX0+1.0, -1.0, -1.0}, 
    {+dX0-1.0, +1.0, -1.0}, 
    {+dX0+1.0, +1.0, +1.0}, 
    {+dX0-1.0, -1.0, +1.0}
};

GLfloat colors[][3] =
{   //    red             green            blue             white
    {1.0, 0.0, 0.0}, {0.0, 1.0, 0.0}, {0.0, 0.0, 1.0}, {1.0, 1.0, 1.0}
};

/*==========================================================*/

void polygon (GLfloat vertices[][3], int a, int b, int c)
{
    /* draw a polygon via list of verticesL */
    glBegin (GL_TRIANGLES);
        glColor3fv  (colors[a]);        // For solid color, set this color only
        glVertex3fv (vertices[a]);
        glColor3fv  (colors[b]);
        glVertex3fv (vertices[b]);
        glColor3fv  (colors[c]);
        glVertex3fv (vertices[c]);
    glEnd ();
    /* draw polygon edges in black */
    glBegin (GL_LINE_LOOP);
        glColor3f (0.0, 0.0, 0.0);
        glVertex3fv (vertices[a]);
        glVertex3fv (vertices[b]);
        glVertex3fv (vertices[c]);
    glEnd ();
}

/*==========================================================*/

void tetrahedron (GLfloat vertices[][3])
{
    /* map verticesL to faces */
    polygon (vertices, 0, 1, 2);
    polygon (vertices, 1, 3, 2);
    polygon (vertices, 0, 3, 1);
    polygon (vertices, 0, 2, 3);
}

/*==========================================================*/

static GLfloat  theta = 0.0;
static GLfloat  trans[] = { dX0, 0.0, 0.0 };
static GLint    axis = 0;

/*==========================================================*/
void mouse (int btn, int state, int x, int y)
{
    static int nFlag = 1;
    void spin (void);

    if (btn == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
    {
        if (nFlag = !nFlag)
            glutIdleFunc (spin);
        else
            glutIdleFunc (NULL);
    }
}

/*==========================================================*/

void display (void)
{
    /* display callback, clear frame buffer and z buffer, 
       rotate figure and draw, swap buffers             */
    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glLoadIdentity ();

    glTranslatef (trans[0], trans[1], trans[2]);
    glRotatef (+theta, 1.0, 0.0, 0.0);      /* x */
    glTranslatef (-trans[0], -trans[1], -trans[2]);

    tetrahedron (verticesL);

    glLoadIdentity ();

    glTranslatef (trans[0], trans[1], trans[2]);
    glRotatef (-theta, 1.0, 0.0, 0.0);      /* x */
    glTranslatef (-trans[0], -trans[1], -trans[2]);

    tetrahedron (verticesR);

    glFlush ();
    glutSwapBuffers ();
}

/*==========================================================*/

void spin (void)
{
    /* Idle callback, rotate tetrahedron 0.25 degrees about selected axis */
    theta += 0.25;
    if (theta > 360.0 )
        theta -= 360.0;
    glutPostRedisplay ();
}

/*==========================================================*/

void myReshape (int w, int h)
{
    double dVal = 4.0;

    glViewport (0, 0, w, h);
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    if (w <= h)
        glOrtho (-dVal, dVal, -dVal * (GLfloat) h / (GLfloat) w, 
                               dVal * (GLfloat) h / (GLfloat) w, -1.5*dVal, 1.5*dVal);
    else
        glOrtho (-dVal * (GLfloat) w / (GLfloat) h, 
                  dVal * (GLfloat) w / (GLfloat) h, -dVal, dVal, -1.5*dVal, 1.5*dVal);
    glMatrixMode (GL_MODELVIEW);
}


/*==========================================================*/

int main (int argc, char **argv)
{
    glutInit (&argc, argv);
    /* need both double buffering and z buffer */
    glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitWindowSize (350, 250);
    glutCreateWindow ("A Pair of Rotating Objects");
    glutReshapeFunc (myReshape);
    glutDisplayFunc (display);
    glutIdleFunc (spin);
    glutMouseFunc (mouse);
    glEnable (GL_DEPTH_TEST); /* Enable hidden-surface-removal */
    glClearColor  (0.5, 0.5, 0.5, 1.0);
    printf ("\nClick the left mouse button to pause/unpause\n");
    printf ("You may resize the OpenGL window\n\n");

    glutMainLoop ();

    return 0;
}

/*==========================================================*/
